/*****************************************************************************
|*
|*  PACKAGE:            AX/HX93X Firmware
|*
|*  VERSION CONTROL:    @(#)usb.h	1.1 98/06/19
|*
|*  COPYRIGHT:          Copyright (c) 1998 Tasking Software B.V.
|*
|*  DESCRIPTION:
|*
|*	USB constant and structure definitions derived from
|*	the USB specification v1.0 (January 19, 1996).
|*
|*	UINT8 and UINT16 data type definitions are introduced
|*	for portability reasons and default to C compiler
|*	conventions for 8/16/32 bit microcontrollers.
|*
|*	The USB protocol assumes Little Endian. For Big Endian
|*	machines (such as the 8x93X using a 251 core), you
|*	must swap the bytes within a 16 and 32 bit value by
|*	defining the macro USB_SWAP_BYTES. For Little Endian
|*	machines this is not needed.
|*
 ****************************************************************************/

#ifndef _USB_H
#define _USB_H

#define USB_SPEC	0x0100	/* USB Spec 1.0 -- Jan 19, 1996 (BCD) */

typedef	unsigned char	UINT8;
typedef unsigned short	UINT16;
typedef	union { UINT16 w; struct { UINT8 msb; UINT8 lsb; } b; } MSBLSB;

/*
 *	Standard Request Codes (Table 9-3 USB spec 1.0)
 */
enum BRequest
{
	GET_STATUS = 0,
	CLEAR_FEATURE,
	GET_STATE,
	SET_FEATURE,
				/* Reserved for future use */
	SET_ADDRESS = 5,  
	GET_DESCRIPTOR,
	SET_DESCRIPTOR,
	GET_CONFIGURATION,
	SET_CONFIGURATION,
	GET_INTERFACE,
	SET_INTERFACE,
	SYNCH_FRAME
};
typedef enum BRequest BREQUEST;


/*
 *	Descriptor Types (Table 9-4 USB spec 1.0)
 */
enum DescrType
{
	DEVICE_DESCR	= 1,
	CONFIG_DESCR,
	STRING_DESCR,
	INTERFACE_DESCR,
	ENDPOINT_DESCR,
	/*
	 *	HID specific descriptor types
	 */
	HID_DESCR 	= 0x21,
	ENTITY_DESCR 	= 0x22,
	HUB_DESCR	= 0x29
};
typedef enum DescrType DESCR_TYPE;

/*
 *	Feature Selectors
 */
enum FeatureSelector
{
	ENDPOINT_STALL = 0,
	DEVICE_REMOTE_WAKEUP
};
typedef enum FeatureSelector FEATURE_SELECTOR;

/*
 *	Recipient Selectors (bit 0-4 in bmRequestType)
 */
enum RecipientSelector
{
	RS_DEVICE = 0,
	RS_INTERFACE,
	RS_ENDPOINT,
	RS_OTHER		/* 4..31 Reserved */
};
typedef enum RecipientSelector RECIPIENT_SELECTOR;
#define USB_RSEL(x)		((x) & 0x1F)

/*
 *	Recipient Types (bit 5-6 in bmRequestType)
 */
enum RecipientType
{
	RT_STANDARD = 0,
	RT_CLASS,
	RT_VENDOR,
	RT_RESERVED
};
typedef enum RecipientType RECIPIENT_TYPE;
#define USB_RTYPE(x)		(((x) & 0x60) >> 5)

/*
 *	Data Transfer Direction (bit 7 in bmRequestType)
 */
enum DataXferDir
{
	DT_HOST_TO_DEVICE = 0,
	DT_DEVICE_TO_HOST
};
typedef enum DataXferDir DATA_XFER_DIR;
#define USB_DIRECTION(x)	(((x) & 0x80) >> 7)
#define USB_DEVICE_TO_HOST(x)	((x) & 0x80)		/* shortcut, only 1 bit */

/*
 *	String indexes
 */
enum StringIndex
{
	LANGUAGE_IDX = 0,
	MANUFACTURER_IDX,
	PRODUCT_IDX,
	SERIAL_IDX,
	CONFIGURATION_IDX,
	INTERFACE_IDX
};
typedef enum StringIndex STRING_INDEX;

/* 
 *	Endpoint Transfer Type (bit 0-1 in bmAttributes
 */
enum TransferType
{
	TT_CONTROL = 0,
	TT_ISOCHRONOUS,
	TT_BULK,
	TT_INTERRUPT
};
typedef enum TransferType TRANSFER_TYPE;

#define EP_IN(address)	((address & 0x7) | 0x80)
#define EP_OUT(address)	(address & 0x7)
#define MAXPWR_mA(mA)	((mA >> 1) & 0xff)
#define PWRON2PWRGOOD(ms) ((ms >> 1) & 0xff)

/*
 *	Hub Change Field (bit 0-1 in wHubChange)
 */
enum HubChangeField
{
	C_HUB_LOCAL_POWER = 0,
	C_HUB_OVER_CURRENT
};
typedef enum HubChangeField HUB_CHANGE_FIELD;

/*
 *	Hub Class Feature Selectors
 */
enum HubClassFeatureSelectors
{
	PORT_CONNECTION = 0,
	PORT_ENABLE,
	PORT_SUSPEND,
	PORT_OVER_CURRENT,
	PORT_RESET,
	/* 5-7 Reserved */
	PORT_POWER = 8,
	PORT_LOW_SPEED,
	/* 10-15 Reserved */
	C_PORT_CONNECTION = 16,
	C_PORT_ENABLE,
	C_PORT_SUSPEND,
	C_PORT_OVER_CURRENT,
	C_PORT_RESET
};
typedef enum HubClassFeatureSelectors HUB_CLASS_FEATURE_SELECTORS;

/*
 *	Encoded Hub Port Control Command (bit 0-2 HPCON)
 */
enum HubPortControlCommand
{
	DISABLE_PORT_CMD = 0,
	ENABLE_PORT_CMD,
	RESET_ENABLE_PORT_CMD,
	SUSPEND_PORT_CMD,
	RESUME_PORT_CMD
};
typedef enum HubPortControlCommand HUB_PORT_CONTROL_COMMAND;

/*
 *	 USB Device Request structure
 */
struct DeviceRequest
{
	UINT8 	bmRequestType;		/* bitmap: recipient selector/type/direction */
	UINT8 	bRequest;		/* cannot use 16-bit BREQUEST enum type! */
	MSBLSB	wValue;			/* corresponding value (varies with request) */
	MSBLSB	wIndex;			/* corresponding index or offset */
	MSBLSB	wLength;		/* number of bytes to xfer when in data phase */
};
typedef struct DeviceRequest COMMAND;

/*
 *	 USB Device Request macro's
 */
#define bDescriptortype		command.wValue.b.msb
#define bDescriptorIndex	command.wValue.b.lsb
#define wvalue			command.wValue.w
#define bmRequestType		command.bmRequestType
#define bRequest		command.bRequest
#define blsbLength		command.wLength.b.lsb
#define bmsbLength		command.wLength.b.msb
#define wlength			command.wLength.w
#define blsbIndex		command.wIndex.b.lsb
#define bmsbIndex		command.wIndex.b.msb
#define windex			command.wIndex.w


/*
 *	USB Device Descriptor structure
 */
struct Device
{
	UINT8 	bLength;		/* size of descr in bytes */
	UINT8 	bDescriptorType;	/* DEVICE: 8-bit DESCR_TYPE value */
	UINT16 	bcdUSB;			/* USB spec release number (BCD) */
	UINT8 	bDeviceClass;		/* device class (assigned by USB) */
	UINT8 	bDeviceSubClass;	/* device subclass (assigned by USB) */
	UINT8 	bDeviceProtocol;	/* protocol code (assigned by USB) */
	UINT8 	wMaxPacketSize0;	/* maximum packet size (endpoint zero) */
	UINT16 	idVendor;		/* vendor id (USB: 0x0628 for TASKING) */
	UINT16 	idProduct;		/* your (manufacturer) product id */
	UINT16 	bcdDevice;		/* device release number (BCD format) */
	UINT8 	iManufacturer;		/* index string: name of your company */
	UINT8 	iProduct;		/* index string: name of your product */
	UINT8 	iSerialNumber;		/* index string: device serial number */
	UINT8 	bNumConfigurations;	/* number of possible configurations */
};
typedef struct Device DEVICE_DESCRIPTOR;

/*
 *	USB HUB Device Descriptor structure
 */
struct HubDescr
{
        UINT8	bLength;		/* size of descr in bytes */
        UINT8	bDescriptorType;	/* DEVICE: 8-bit DESCR_TYPE value */
        UINT8	bNumPorts;              /* Number of downstream ports */
        UINT16	wHubCharacteristics;	/* Power switch mode */
        UINT8	bPwrOn2PwrGood;		/* Time power to stabilize */
        UINT8	bHubConrCurrent;	/* Max. current requirements of hub */
        UINT8	bDeviceRemovable;       /* Indicates removable device attached */
        UINT8	bPortPwrCtrlMask;       /* Toggle Ports respond to Ganged Power Commands */
};
typedef struct HubDescr HUB_DESCRIPTOR;


/*
 * 	Configuration Descriptor structure
 */
struct Config
{
        UINT8	bLength;		/* size of descr in bytes */
        UINT8 	bDescriptorType;	/* CONFIGURATION: 8 bit DESCR_TYPE value */
        UINT16 	wTotalLength;		/* total length of data returned */
        UINT8 	bNumInterfaces;		/* number of interfaces supported */
        UINT8 	bConfigurationValue;	/* Set Configuration selection */
        UINT8 	iConfiguration;		/* index string: configuration */
        UINT8 	bmAttributes;		/* configuration characteristics */
        UINT8 	MaxPower;		/* maximum power consumption */
};
typedef struct Config 		CONFIG_DESCRIPTOR;

/*
 *	Valid Configuration Characteristics:
 */
#define USB_CONFIG_BUS_POWERED		0x80
#define USB_CONFIG_SELF_POWERED		0x40
#define USB_CONFIG_REMOTE_WAKEUP	0x20

/*
 *	Interface Descriptor structure
 */
struct Interface
{
        UINT8 	bLength;	   	/* size of the descriptor in bytes */
	UINT8 	bDescriptorType;	/* INTERFACE: 8 bit DESCR_TYPE value */
        UINT8 	bInterfaceNumber;	/* number of the interface */
        UINT8 	bAlternateSetting;	/* select alternate setting */
        UINT8 	bNumEndpoints;		/* number of endpoints used */
        UINT8 	bInterfaceClass;	/* class code (assigned by USB) */
        UINT8 	bInterfaceSubClass;	/* subclass code (assigned by USB) */
        UINT8 	bInterfaceProtocol;	/* protocol code (assigned by USB) */
        UINT8 	iInterface;		/* index string: name of interface */
};
typedef struct Interface 	INTERFACE_DESCRIPTOR;

/*
 *	End Point Descriptor structure
 */
struct Endpoint
{
        UINT8 	bLength;		/* size of the descriptor in bytes */
        UINT8 	bDescriptorType;	/* INTERFACE: 8 bit DESCR_TYPE value */
        UINT8 	bEndpointAddress;	/* endpoint address */
        UINT8 	bmAtrributes;		/* configuration characteristics */
        UINT16 	wMaxPacketSize;		/* Maximum packet size */
        UINT8 	bInterval;		/* polling interval */
};
typedef struct Endpoint		ENDPOINT_DESCRIPTOR;

/*
 *	End Point Input Descriptor macro
 */
#define EPIN_DESCR( n, type, maxsz, pol )						\
{	sizeof(ENDPOINT_DESCRIPTOR),	/* size of this descriptor in bytes */		\
	ENDPOINT_DESCR,			/* endpoint descriptor type */			\
	EP_IN(n),			/* EP address n, in */				\
        type,				/* 00-Control, 1-ISO, 2-Bulk,3-Interrupt */	\
	SWAP16(maxsz),			/* max packet size */				\
        pol				/* polling interval */				\
}

/*
 *	End Point Output Descriptor macro
 */
#define EPOUT_DESCR( n, type, maxsz, pol )						\
{	sizeof(ENDPOINT_DESCRIPTOR),	/* size of this descriptor in bytes */		\
	ENDPOINT_DESCR,			/* endpoint descriptor type */			\
	EP_OUT(n),			/* EP address n, out */				\
        type,				/* 00-Control, 1-ISO, 2-Bulk,3-Interrupt */	\
       	SWAP16(maxsz),			/* max packet size */				\
        pol				/* polling interval */				\
}

/*
 *	Request types
 */
#define	SET_STANDARD_DEVICE_COMMAND	0x00
#define	GET_STANDARD_DEVICE_COMMAND	0x80
#define	SET_STANDARD_INTERFACE_COMMAND	0x01
#define	GET_STANDARD_INTERFACE_COMMAND	0x81
#define	SET_STANDARD_ENDPOINT_COMMAND	0x02
#define	GET_STANDARD_ENDPOINT_COMMAND	0x82
#define	SET_STANDARD_OTHER_COMMAND	0x03
#define	GET_STANDARD_OTHER_COMMAND	0x83
#define	SET_CLASS_DEVICE_COMMAND	0x20
#define	GET_CLASS_DEVICE_COMMAND	0xa0
#define	SET_CLASS_INTERFACE_COMMAND	0x21
#define	GET_CLASS_INTERFACE_COMMAND	0xa1
#define	SET_CLASS_ENDPOINT_COMMAND	0x22
#define	GET_CLASS_ENDPOINT_COMMAND	0xa2
#define	SET_CLASS_OTHER_COMMAND		0x23
#define	GET_CLASS_OTHER_COMMAND		0xa3
#define	SET_VENDOR_DEVICE_COMMAND	0x40
#define	GET_VENDOR_DEVICE_COMMAND	0xc0
#define	GET_VENDOR_INTERFACE_COMMAND	0x41
#define	SET_VENDOR_INTERFACE_COMMAND	0xc1
#define	SET_VENDOR_ENDPOINT_COMMAND	0x42
#define	GET_VENDOR_ENDPOINT_COMMAND	0xc2
#define	SET_VENDOR_OTHER_COMMAND	0x43
#define	GET_VENDOR_OTHER_COMMAND	0xc3

/*
 *	Stall Masks
 */
#define	CLEAR_IN_STALL_MASK		0xBF
#define	CLEAR_OUT_STALL_MASK		0x7F
#define	SET_IN_STALL_MASK		0x40
#define	SET_OUT_STALL_MASK		0x80

#define EP0_MAX_PACKET_SIZE		0x08
#define NULL_DATA_PACKET		0x00

/*
 *	USB protocol assumes Little Endian. For Big Endian machines (such as the
 *	8x93X using a 251 or 8051 core), you must swap the bytes within a 16 bit value:
 */
#ifdef USB_SWAP_BYTES
#define SWAP16(x)	((((x) >> 8) & 0x00FF) | (((x) << 8) & 0xFF00))
#else
#define	SWAP16(x)	(x)
#endif

/*
 *	USB state machine stages
 */
enum UsbStage
{
	SETUP_STAGE = 0,
	WRITE_STAGE,
	READ_STAGE
};
typedef enum UsbStage USB_STAGE;

#endif /* _USB_H */






